myUI = ...
myUI.tostring = function(self) return self.toc.Name .. " - v" .. self.toc.Version end

local _context = UI.CreateContext(myUI.identifier); _context:SetSecureMode("restricted")
local _modules = {}
local L = myUI.data.language
local ifnil = myUI.Utility.ifnil

local tinsert = table.insert
local tremove = table.remove

--dump(myUI)

function myUI:Modules()
	return pairs(_modules)
end

function myUI:RegisterModule(module)
	_modules[module.identifier] = module	
	module.tostring = function(self) return self.toc.Name .. " - v" .. self.toc.Version end	
end


-- event hooking/unhooking (helpers for when an event may need to be unhooked by an addon)

function myUI:HookEvent(eventTable, func, addon, uniqueidentifier)
	if (not addon.data.__hooks) then addon.data.__hooks = {} end
	if (not addon.data.__hooks[eventTable]) then addon.data.__hooks[eventTable] = {} end
	
	if (addon.data.__hooks[eventTable][uniqueidentifier]) then
		-- Error !  this event is already hooked by this addon using this uniqueidentifier
		-- Not sure of the correct way to throw this error so we'll just silently fail for now
	else
		tinsert(eventTable, {func, addon.identifier, uniqueidentifier})
	
		addon.data.__hooks[eventTable][uniqueidentifier] = true
		return true
	end
	
end

function myUI:UnhookEvent(eventTable, addon, uniqueidentifier)
	if (addon.data.__hooks) and (addon.data.__hooks[eventTable]) then
		local removeAt
		local hooktable = addon.data.__hooks[eventTable]
		for i, hook in ipairs(eventTable) do
			if (hooktable[hook[3]]) then
				removeAt = i
				break
			end	
		end
		if (removeAt) then 
			tremove(eventTable, removeAt) 
			return true 
		end
	end
end

-- myUI Event support
local function _subscribe(event, uniqueid, module, func)
	if (nil == func) or (nil == uniqueid) then return false end
	if (nil == event._subscribers[uniqueid]) then
		event._subscribers[uniqueid] = {f = func, m = module}
		return true
	else
		return false
	end
end

local function _unsubscribe(event, uniqueid)
	if (nil == uniqueid) then return false end
	event._subscribers[uniqueid] = nil
	return true
end

local function _issubscribed(event, uniqueid)
	if (nil == uniqueid) then return false end
	return (nil ~= event._subscribers[uniqueid])
end

local _eventPrototype = {	
	new = function(self)
		return {
			_subscribers = {},
			Subscribe =_subscribe,
			Unsubscribe = _unsubscribe,
			IsSubscribed = _issubscribed
		} 	
	end
}



myUI.Events = {
	AnimationUpdate = _eventPrototype:new(), -- function(self, now, elapsed)	
	UnitUpdate = _eventPrototype:new(),
	SlowUpdate = _eventPrototype:new(),
}




-- update related events

local _animationListeners = myUI.Events.AnimationUpdate._subscribers
local _animationThrottle = 1/20 -- defaults to 20 times a second
local _lastAnimationUpdate = 0

local _unitUpdateListeners = myUI.Events.UnitUpdate._subscribers
local _unitUpdateThrottle = 1/10
local _lastUnitUpdate = 0

local _slowUpdateListeners = myUI.Events.SlowUpdate._subscribers
local _slowUpdateThrottle = 1/2
local _lastSlowUpdate = 0

local function _update()

	local now = Inspect.Time.Frame()
	
	local elapsed = now - _lastAnimationUpdate	
	if (elapsed >= _animationThrottle) then
		_lastAnimationUpdate = now
		for _, s in pairs(_animationListeners) do s.f(s.m, now, elapsed) end
	end
	
	elapsed = now - _lastUnitUpdate
	if (elapsed >= _unitUpdateThrottle) then
		_lastUnitUpdate = now
		for _, s in pairs(_unitUpdateListeners) do s.f(s.m, now, elapsed) end
	end
	
	
end


function myUI:GetAnimationSpeed()
	return myUI_Account.Global.AnimationSpeed
end

function myUI:SetAnimationSpeed(fps)
	myUI_Account.Global.AnimationSpeed = fps
	_animationThrottle = 1/fps
end

function myUI:GetUnitUpdateInterval()
	return myUI_Account.Global.UnitUpdateSpeed
end

function myUI:SetUnitUpdateInterval(fps)
	myUI_Account.Global.UnitUpdateSpeed = fps
	_unitUpdateThrottle = 1/fps
end
function myUI:GetSlowUpdateInterval()
	return myUI_Account.Global.SlowUpdateSpeed
end
function myUI:SetSlowUpdateInterval(fps)
	myUI_Account.Global.SlowUpdateSpeed = fps
	_slowUpdateThrottle = 1/fps
end


function myUI:GetSlashCommandUsage()
	local r = {}
	
	table.insert(r, L["slash use"])
	
	table.insert(r, "        config - " .. L["config"])
	table.insert(r, "        profile load [" .. L["name"] .. "] - " .. L["profile load"])
	table.insert(r, "        version, ver - " .. L["version"])
	table.insert(r, "        modules, module list - " .. L["modules"])
		
	for _, m in pairs(_modules) do
		if (m.GetSlashCommandDescriptions) then
			local lines = m:GetSlashCommandDescriptions()
			if (lines) then
				for _, ln in ipairs(lines) do
					tinsert(r, "        " .. ln)
				end
			end
		end
	end
	return r
end

local function _command(commandline)
	if (commandline == "config") then
		myUI:ShowConfiguration()
	elseif (string.find(commandline, "profile load ") == 1) then
		local profile = myUI.Utility.trim(string.sub(commandline, 14))
		if (string.len(profile) > 0) then
			myUI:SetCurrentProfile(profile, true)
		end
	elseif (commandline == "version") or (commandline == "ver") then
		print(L["Version "] .. myUI.toc.Version)
	elseif (commandline == "modules") or (commandline == "modules list") then
		print(L["Modules"] .. ":")
		for _, m in pairs(_modules) do
			print("        " .. m.toc.Name)
		end
	else
		local handled
		for _, m in pairs(_modules) do
			if (m.HandleSlashCommand) then
				handled = m:HandleSlashCommand(commandline)
			end
			if (handled) then break end
		end
		
		if (not handled) then
			local lines = myUI:GetSlashCommandUsage()
			for _, ln in ipairs(lines) do print(ln) end
		end		
	end

end





local _reloadBox, _setupReloadBox


function myUI:ReloadUI()
	_reloadBox:ClearAll()
	_reloadBox:SetWidth(400)
	_reloadBox:SetHeight(160)
	_reloadBox:SetPoint("CENTER", UIParent, "CENTER")
end

-- frame creation
local _frameCount = 0
local _text = {} -- references to all text frames created
local _font
local _fontSize = 16

local function _setSize(frame, cx, cy)
	if (cx) then
		cy = ifnil(cy, cx)
		frame:SetWidth(cx)
		frame:SetHeight(cy)
	end
end

function myUI:CreateFrame(ftype, parent)
	parent = ifnil(parent, _context)
	
	_frameCount = _frameCount + 1
	local f
	
	if (string.find(ftype, "myUI_")) then
		f = myUI:CreateWidget(ftype, parent)
	else	
		f = UI.CreateFrame(ftype, myUI.identifier .. "." .. ftype .. "." .. tostring(_frameCount), parent)
	end
	f:SetSecureMode("normal")
	
	f.SetSize = _setSize
	
	if (ftype == "Text") then 
		tinsert(_text, f) 
		f:SetFont(_font.src, _font.file)
		f:SetFontSize(_fontSize)
	end
	
	return f
end

function myUI:SetGlobalFont(font, size)
	if (myUI_Account.Global.Fonts[font]) then
		myUI_Account.Profiles[myUI:GetCurrentProfile()].GlobalFont = font
		if (size) then 
			myUI_Account.Profiles[myUI:GetCurrentProfile()].GlobalFontSize = size 
			_fontSize = size
		end
		
		_font = myUI_Account.Global.Fonts[font]
		
		for _, text in ipairs(_text) do
			if (not text.customFont) then
				text:SetFont(_font.src, _font.file)				
			end
			if (not text.customFontSize) then
				text:SetFontSize(_fontSize)
			end
		end
		
		return true
	end
end



function myUI:KnownGameIcons()
	return pairs(myUI_Account.Global.Icons)
end

function myUI:ScanForGameIcons()
	local abilities = Inspect.Ability.List()
	if (abilities) then
		abilities = Inspect.Ability.Detail(abilities)
		if (abilities) then
			for _, detail in pairs(abilities) do
				if (detail.icon) then
					myUI_Account.Global.Icons[detail.icon] = "Rift"
				end
			end
	
		end
	end
	local items = Inspect.Item.List()
	if (items) then
		items = Inspect.Item.Detail(items)
		if (items) then
			for _, detail in pairs(items) do
				if (detail.icon) then
					myUI_Account.Global.Icons[detail.icon] = "Rift"
				end
			end
		end
	end
end

local function _loaded()	
	_setupReloadBox()

	_animationThrottle = 1 / myUI_Account.Global.AnimationSpeed
	_unitUpdateThrottle = 1/ myUI_Account.Global.UnitUpdateSpeed
	_slowUpdateThrottle = 1/myUI_Account.Global.SlowUpdateSpeed
	
	myUI:ScanForGameIcons()
	
	local profileName = myUI:GetCurrentProfile()
	
	_font = myUI_Account.Global.Fonts[myUI_Account.Profiles[profileName].GlobalFont]
	if (not _font) then _font = myUI_Account.Global.Fonts["Rift Default"] end
	
	if (myUI_Account.Profiles[profileName].GlobalFontSize) then
		_fontSize = myUI_Account.Profiles[profileName].GlobalFontSize
	end
	print(myUI.toc.Name .. " " .. L[" loaded, Version "] .. myUI.toc.Version)
	
	for _, module in pairs(_modules) do
		if (module.Initialize) then module:Initialize() end
		print(module.toc.Name .. " " .. L[" loaded, Version "] .. module.toc.Version)		
	end
	
	for _, module in pairs(_modules) do
		if (module.PostInitialize) then
			module:PostInitialize()
		end
	end
	
	myUI:SetCurrentProfile(profileName)	
end





_setupReloadBox = function()
	_reloadBox = UI.CreateFrame("RiftWindow", "myUI_ReloadUI", _context)
	_reloadBox:SetSecureMode("normal")
	_reloadBox:SetTitle(myUI.toc.Name)
	_reloadBox:SetPoint("RIGHTCENTER", UIParent, "LEFTCENTER", -100, 0) -- offscreen
	_reloadBox:SetLayer(myUI.Layers.Configuration + 100)
	
	local txt = UI.CreateFrame("Text", "myUI_ReloadUI.Text", _reloadBox)
	txt:SetWordwrap(true)
	txt:SetPoint("TOPCENTER", _reloadBox, "TOPCENTER", 0, 60)
	txt:SetText(L["reloadui"])
	
	local btn = UI.CreateFrame("RiftButton", "myUI_ReloadUI.Button", _context)
	btn:SetText(L["ok"])
	btn:SetPoint("BOTTOMCENTER", _reloadBox, "BOTTOMCENTER", 0, -16)
	btn:SetLayer(_reloadBox:GetLayer()+10)
	btn:SetSecureMode("restricted")
	btn.Event.LeftPress = "reloadui"
	
end

-- setup framework events

table.insert(Event.Addon.Startup.End, {_loaded, myUI.identifier, "_loaded"})
table.insert(Event.System.Update.End, {_update, myUI.identifier, "_update"})
table.insert(Command.Slash.Register(myUI.identifier), {_command, myUI.identifier, "_command"})

